/*
 * Project: VizKit
 * Version: 3.3.4
 
 * Date: 20130804
 * File: VisualFontManager.cpp
 *
 */

/***************************************************************************

Copyright (c) 2004-2013 Heiko Wichmann (http://www.imagomat.de/vizkit)


This software is provided 'as-is', without any expressed or implied warranty. 
In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; 
   you must not claim that you wrote the original software. 
   If you use this software in a product, an acknowledgment 
   in the product documentation would be appreciated 
   but is not required.

2. Altered source versions must be plainly marked as such, 
   and must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.

 ***************************************************************************/

#include "VisualFontManager.h"
#include "VisualErrorHandling.h"
#include "VisualConfiguration.h"
#include "VisualGraphicsCore.h"

#include <algorithm> // sort on vector

#if TARGET_OS_WIN
#include "win/resource.h"
#include "VisualFile.h"
#endif

#if TARGET_OS_MAC
#import <Cocoa/Cocoa.h>
#endif


using namespace VizKit;

// file type of VisualFontManager.cpp is set to "Objective-C++ source"

VisualFontManager* VisualFontManager::theVisualFontManager = NULL;  // singleton


#if TARGET_OS_WIN
	uint8 fontIsInstalledBool;
#endif



#if TARGET_OS_WIN

int CALLBACK EnumFontFamExProc(
				ENUMLOGFONTEX *lpelfe,    // logical-font data
				NEWTEXTMETRICEX *lpntme,  // physical-font data
				DWORD FontType,           // type of font
				LPARAM lParam             // application-defined data
)
{
	//writeLog((char *)lpelfe->elfLogFont.lfFaceName);

	fontIsInstalledBool = 1;
	return 1;
}


uint8 VisualFontManager::fontIsInstalled(HDC currHdc, const char* const fontName) 
{

	LOGFONT lf;

	lf.lfFaceName[0]='\0';
	strcpy(lf.lfFaceName, fontName);
	fontIsInstalledBool = 0;
	EnumFontFamiliesEx(currHdc, &lf, (FONTENUMPROC) EnumFontFamExProc, 0, 0);
	if (fontIsInstalledBool == 1) {
		return 1;
	} else {
		return 0;
	}

}

#endif


VisualFontManager::VisualFontManager() 
{
    //
}


VisualFontManager::~VisualFontManager() 
{
#if TARGET_OS_WIN
	std::vector<fontItem*>::iterator it2;
	for (it2 = this->FontVector.begin(); it2 != this->FontVector.end(); it2++) {
		delete *it2;
		*it2 = NULL;
	}
	FontVector.clear();
#endif
}


VisualFontManager* VisualFontManager::getInstance() 
{
    if (theVisualFontManager == NULL) 
    {
		theVisualFontManager = new VisualFontManager;
    }
	return theVisualFontManager;
}


void VisualFontManager::dispose() 
{
	if (theVisualFontManager != NULL) 
    {
		delete theVisualFontManager;
		theVisualFontManager = NULL;
	}
}


bool VisualFontManager::fontIsAvailableAsVisualizerResource(const char* const fontName) {
	bool fontIsAvailable = false;
	if (strcmp(fontName, "Devroye") == 0) {
		fontIsAvailable = true; 
	}
	return fontIsAvailable;
}


#if TARGET_OS_MAC
bool VisualFontManager::fontIsAvailable(const char* const fontNameUTF8String)
{
    NSString *aFontName = [NSString stringWithCString:fontNameUTF8String encoding:NSUTF8StringEncoding];
    
	Boolean result = false;
    
	NSString* fontNameStr;
	NSArray* fontArray;
	NSEnumerator* e;
    
	NSFontManager* fontManager = [NSFontManager sharedFontManager];
    
	fontArray = [fontManager availableFonts];
	e = [fontArray objectEnumerator];
	while ((fontNameStr = [e nextObject])) {
		if ([fontNameStr isEqualToString:aFontName]) {
			result = true;
		}
		/* NSLog(fontNameStr); */
	}
	
	return result;
    
}


OSStatus VisualFontManager::activateCocoaFont(const char* const fontNameUTF8String) 
{    
	OSStatus aStatus = noErr;
    
	// assume Font is available as resource of bundle
	NSBundle* anAppBundle = [NSBundle bundleWithIdentifier:[NSString stringWithCString:VisualConfiguration::visualPluginDomainIdentifier() encoding:NSUTF8StringEncoding]];
	NSString* aFontPath = [anAppBundle resourcePath];
    aFontPath = [aFontPath stringByAppendingPathComponent: @"DEVROYE_.otf"];
    
    NSURL *aFontURL = [NSURL fileURLWithPath:aFontPath isDirectory:NO];
    if (aFontURL)
    {
        CFErrorRef *anError = NULL;
        bool aSuccess = CTFontManagerRegisterFontsForURL((CFURLRef)aFontURL, kCTFontManagerScopeProcess, anError);
        if (aSuccess == false)
        {
            aStatus = 1001;
        }
    }
    
	return aStatus;
    
}

#endif


VisualFontActivationResult VisualFontManager::activateFont(const char* const fontName) {
	VisualFontActivationResult result = fontActivationUndefined;

	theVisualFontManager = VisualFontManager::getInstance();
#if TARGET_OS_MAC
	OSStatus status = noErr;
	bool fontIsAvailable = theVisualFontManager->fontIsAvailable(fontName);
	if (!fontIsAvailable) {
		status = theVisualFontManager->activateCocoaFont(fontName);
	} else {
		result = fontActivationFontAlreadyAvailable;
	}
    
	if (status == noErr) 
    {
		result = fontActivationNoError;
	} 
    else if (result == fontActivationUndefined) 
    {
		result = fontActivationUnableToActivateFont;
	}
#endif

#if TARGET_OS_WIN
	VisualGraphicsCore* theVisualGraphicsCore = VisualGraphicsCore::getInstance();
	uint8 fontIsInstalledBool = theVisualFontManager->fontIsInstalled(theVisualGraphicsCore->getWindowDC(), fontName);
	if (fontIsInstalledBool == 0) 
	{
		void* fontData = NULL;
		uint32 sizeOfFontResource = 0;
		bool success = VisualFile::getDataOfResource(IDR_DEVROYEFONT, "FONTRESOURCE", &fontData, sizeOfFontResource);
		if (success) {
			HANDLE fontResHandle = NULL;
			DWORD numberOfFonts = 1;
			fontResHandle = AddFontMemResourceEx(fontData, sizeOfFontResource, 0, &numberOfFonts);
			if (!fontResHandle || numberOfFonts == 0) {
				char errLog[512];
				sprintf(errLog, "AddFontMemResourceEx failed in file: %s (line: %d) [%s])", __FILE__, __LINE__, __FUNCTION__);
				writeLog(errLog);
				result = fontActivationUnableToActivateFont;
			} else {
				// system made its own copy
				DeleteObject(fontData);
				result = fontActivationNoError;
			}
		} else {
			char errLog[512];
			sprintf(errLog, "Err: Unable to get resource data in file: %s (line: %d) [%s])", __FILE__, __LINE__, __FUNCTION__);
			writeLog(errLog);
			result = fontActivationUnableToActivateFont;
		}
	} else {
		result = fontActivationFontAlreadyAvailable;
	}
#endif
    
	return result;
}


#if TARGET_OS_WIN

struct CompareFontNamesAscending {

	bool operator()(const fontItem* x, const fontItem* y) {
		if (strcmp(x->name, y->name) > 0) {
			return false;
		} else {
			return true;
		}
	}

};


int CALLBACK VisualFontManager::EnumFontFamExProc2(
				ENUMLOGFONTEX *lpelfe,    // logical-font data
				NEWTEXTMETRICEX *lpntme,  // physical-font data
				DWORD FontType,           // type of font
				LPARAM lParam             // application-defined data
) {

	fontItem* aFontItem;

	VisualFontManager* theVisualFontManager = VisualFontManager::getInstance();
	
	if (!(FontType & RASTER_FONTTYPE)) {
	//} else {
		aFontItem = new fontItem;
		strcpy(aFontItem->name, (char*)lpelfe->elfFullName);
		//strcpy(aFontItem->name, (char*)lpelfe->elfLogFont.lfFaceName);
		
		theVisualFontManager->FontVector.push_back(aFontItem);
	}

	return 1;
}


void VisualFontManager::resetFontVectorIterIndex() {
	this->fontVectorIterIdx = 0;
}


void VisualFontManager::gatherFonts(void) {
	std::vector<fontItem*>::iterator it;
	LOGFONT lf;

	for (it = this->FontVector.begin(); it != this->FontVector.end(); it++) {
		delete *it;
		*it = NULL;
	}
	this->FontVector.clear();

	ZeroMemory(&lf, sizeof(lf));
	lf.lfCharSet = ANSI_CHARSET;
	lf.lfFaceName[0] = '\0';

	VisualGraphicsCore* theVisualGraphicsCore = VisualGraphicsCore::getInstance();
	EnumFontFamiliesEx(theVisualGraphicsCore->getWindowDC(), &lf, (FONTENUMPROC) VisualFontManager::EnumFontFamExProc2, 0, 0);
	
}


void VisualFontManager::sortFontsByName() {
	std::sort(this->FontVector.begin(), this->FontVector.end(), CompareFontNamesAscending());
}


void VisualFontManager::addFont(const char* const fontName) {
	fontItem* aFontItem = new fontItem;
	strcpy(aFontItem->name, fontName);
	this->FontVector.push_back(aFontItem);
}


bool VisualFontManager::getNextFontName(char* fontName) {
	if (this->fontVectorIterIdx < this->FontVector.size()) {
		strcpy (fontName, this->FontVector[this->fontVectorIterIdx]->name);
		this->fontVectorIterIdx++;
		return true;
	} else {
		return false;
	}
}


#endif
