/*
 * Project: VizKit
 * Version: 3.3.4
 
 * Date: 20130804
 * File: VisualConfiguration.cpp
 *
 */

/***************************************************************************

Copyright (c) 2004-2013 Heiko Wichmann (http://www.imagomat.de/vizkit)


This software is provided 'as-is', without any expressed or implied warranty. 
In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; 
   you must not claim that you wrote the original software. 
   If you use this software in a product, an acknowledgment 
   in the product documentation would be appreciated 
   but is not required.

2. Altered source versions must be plainly marked as such, 
   and must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.

 ***************************************************************************/

#include "VisualConfiguration.h"
#include "VisualErrorHandling.h"
#include "VisualCustomization.h"


#if TARGET_OS_MAC
#include <CoreFoundation/CFString.h> // CFStringRef
#endif

#if TARGET_OS_WIN
#include <windows.h>
#endif

#include <string> // strlen

using namespace VizKit;



VisualConfiguration* VisualConfiguration::theVisualConfiguration = NULL;



const char* const VisualConfiguration::visualPluginDomainIdentifier()
{
    return PluginDomainIdentifier;
}


const char* const VisualConfiguration::visualizerPluginIdentifierName()
{
    return PluginIdentifierName;
}


const uint8 VisualConfiguration::majorReleaseNumber()
{
    return MajorReleaseNumber;
}


const uint8 VisualConfiguration::minorReleaseNumber()
{
    return MinorReleaseNumber;
}


const uint8 VisualConfiguration::releaseRevisionNumber()
{
    return ReleaseRevisionNumber;
}


void VisualConfiguration::prepareVisualizerShowNameUnicode() 
{
	
	const char* const aVisualizerShowNameUTF8 = ShowName;

#if TARGET_OS_WIN

	DWORD dwFlags = 0;
	int aVisualizerShowNameDestinationSize = 0;
	int aVisualizerShowNameRequiredSize = MultiByteToWideChar(CP_UTF8, dwFlags, aVisualizerShowNameUTF8, strlen(aVisualizerShowNameUTF8), (LPWSTR)(this->visualizerShowName), aVisualizerShowNameDestinationSize);
	if (aVisualizerShowNameRequiredSize > 0) 
	{
		aVisualizerShowNameDestinationSize = aVisualizerShowNameRequiredSize;
		this->visualizerShowName = (uint16*)malloc((aVisualizerShowNameDestinationSize + 1) * sizeof(uint16));
		int aSizeConverted = MultiByteToWideChar(CP_UTF8, dwFlags, aVisualizerShowNameUTF8, strlen(aVisualizerShowNameUTF8), (LPWSTR)(this->visualizerShowName), aVisualizerShowNameDestinationSize);
		if (aSizeConverted == 0)
		{
			writeLog("ERR: aSizeConverted == 0 when converting visualizer show name from UTF8 to UTF16");
		}
		this->visualizerShowName[aVisualizerShowNameDestinationSize] = L'\0';
		this->numberOfCharactersOfVisualizerShowName = aVisualizerShowNameDestinationSize;
	}
	else
	{
		writeLog("ERR: aVisualizerShowNameRequiredSize == 0");
	}

#endif

#if TARGET_OS_MAC
    
    Boolean isExternalRepresentation = false;
	CFStringRef aVisualizerShowNameCFStringRef = CFStringCreateWithBytes(kCFAllocatorDefault, (const UInt8*)aVisualizerShowNameUTF8, strlen(aVisualizerShowNameUTF8), kCFStringEncodingUTF8, isExternalRepresentation);
    if (aVisualizerShowNameCFStringRef)
    {
        this->numberOfCharactersOfVisualizerShowName = CFStringGetLength(aVisualizerShowNameCFStringRef);
        if (this->numberOfCharactersOfVisualizerShowName > 0)
        {
            this->visualizerShowName = (UniChar*)malloc((size_t)this->numberOfCharactersOfVisualizerShowName * sizeof(UniChar));
            CFStringGetCharacters(aVisualizerShowNameCFStringRef, CFRangeMake(0, this->numberOfCharactersOfVisualizerShowName), this->visualizerShowName);
        }
        else
        {
            writeLog("ERR: numberOfCharactersOfVisualizerShowName == 0");
        }
        CFRelease(aVisualizerShowNameCFStringRef);
    }
    else
    {
        writeLog("ERR: aVisualizerShowNameCFStringRef is NULL");
    }

#endif
}


VisualConfiguration::VisualConfiguration() 
{
	prepareVisualizerShowNameUnicode();
}


VisualConfiguration::~VisualConfiguration() {
	if (theVisualConfiguration->visualizerShowName != NULL) 
    {
		free(theVisualConfiguration->visualizerShowName);
	}
}


VisualConfiguration* VisualConfiguration::getInstance() 
{
    if (theVisualConfiguration == NULL) 
    {
		theVisualConfiguration = new VisualConfiguration;
    }
	if (theVisualConfiguration == NULL) 
    {
		writeLog("ERR: init theVisualConfiguration failed");
	}
	return theVisualConfiguration;
}


void VisualConfiguration::dispose() 
{
	if (theVisualConfiguration != NULL) 
    {
		delete theVisualConfiguration;
		theVisualConfiguration = NULL;
	}
}


uint16* VisualConfiguration::getVisualizerShowName(long& numberOfCharactersOfVisualizerShowName)
{
    uint16 *aVisualizerShowName = NULL;
    numberOfCharactersOfVisualizerShowName = 0;
	theVisualConfiguration = VisualConfiguration::getInstance();
    if (theVisualConfiguration)
    {
        numberOfCharactersOfVisualizerShowName = theVisualConfiguration->numberOfCharactersOfVisualizerShowName;
        aVisualizerShowName = theVisualConfiguration->visualizerShowName;
    }
    return aVisualizerShowName;
}


const char* const VisualConfiguration::currentVersionInformationURL() 
{
    return CurrentVersionInformationURL;
}
		

uint8 VisualConfiguration::getBCD(uint8 decimalNumber) 
{
	
	if (decimalNumber > 99) return getBCD(99);
	
	uint8 bcd = 16 * (decimalNumber / 10) + (decimalNumber%10);
	return bcd;
}


uint8 VisualConfiguration::getMajorReleaseNumberAsBCD() 
{
	return VisualConfiguration::getBCD(VisualConfiguration::majorReleaseNumber());
}


uint8 VisualConfiguration::getMinorReleaseNumberAsBCD() 
{
	return VisualConfiguration::getBCD(VisualConfiguration::minorReleaseNumber());
}


uint8 VisualConfiguration::getReleaseRevisionNumber() 
{
	return VisualConfiguration::releaseRevisionNumber();
}
