/*
 * Project: VizKit
 * Version: 3.3.4
 
 * Date: 20130804
 * File: VisualGraphicsCore.h
 *
 */

/***************************************************************************

Copyright (c) 2004-2013 Heiko Wichmann (http://www.imagomat.de/vizkit)


This software is provided 'as-is', without any expressed or implied warranty. 
In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; 
   you must not claim that you wrote the original software. 
   If you use this software in a product, an acknowledgment 
   in the product documentation would be appreciated 
   but is not required.

2. Altered source versions must be plainly marked as such, 
   and must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.

 ***************************************************************************/

#ifndef VisualGraphicsCore_h
#define VisualGraphicsCore_h


#include "VisualTypes.h"
#include "VisualGraphicTypes.h"


#if TARGET_OS_MAC
#include "VisualCustomization.h"
#ifdef __OBJC__
@class VisualView;
#else
struct VisualView;
#endif
#endif

// definitions from "iTunesAPI.h"
#if TARGET_OS_WIN
#ifndef VISUAL_PLATFORM_VIEW
#define VISUAL_PLATFORM_VIEW HWND
#endif
#else // TARGET_OS_MAC
#ifdef __OBJC__
@class NSView;
#else
struct NSView;
#endif
#ifndef VISUAL_PLATFORM_VIEW
#define VISUAL_PLATFORM_VIEW NSView*
#endif
#endif


#if TARGET_OS_WIN
#include <windows.h> // HDC
#endif


namespace VizKit {

	/**
	 * The platform specific core operations of the graphics layer are bundled with this class. 
     * VisualGraphics is the only class that accesses VisualGraphicsCore. 
     * VisualGraphicsCore is a singleton class.
	 */
	class VisualGraphicsCore {

	public:

		/**
		 * Returns a pointer to the instance. 
		 * Initializes the Visual Graphics Core class if neccessary. 
		 * The visual graphics core class is of singleton type.
		 * @return A pointer to the initialized VisualGraphicsCore.
		 */
		static VisualGraphicsCore* getInstance(void);

		/**
		 * Cleans up the Visual Graphics Core singleton class.
		 */	
		static void dispose(void);

		/**
		 * Sets up an OpenGL context of the operating system.
		 * OS windowing system and OpenGL get connected.
		 * @return true on success, false on failure.
		 */
		bool setupContext();
        
        /**
		 * Cleans up the OpenGL graphics context.
		 */
        void cleanupContext();

		/**
		 * Sets the current context to the one that has been set up before with the setupContext() call.
		 * @return errNum: 0 if no error, > 0 error.
		 * @remarks The context must be set back to the one set up initially with each render action in case Cover Flow did change it in the meantime.
		 */
		uint16 setCurrentContext(void);
		
#if TARGET_OS_MAC
        void startRendering(void);
#endif

#if TARGET_OS_WIN
		/**
		 * Returns the device context. Windows-only.
		 * @return The device context. 
		 */
		HDC getWindowDC(void);
#endif

		/**
		 * Keeps a reference to the platform specific view for internal use.
		 * @param aView A platform specific view.
		 * @remarks On Mac VISUAL_PLATFORM_VIEW is defined as NSView*, on Windows it is defined as HWND.
		 */
		void setPlatformView(VISUAL_PLATFORM_VIEW aView);

		/**
		 * Returns the platform specific view for rendering.
		 * @return The platform specific view for rendering.
		 * @remarks On Mac VISUAL_PLATFORM_VIEW is defined as NSView*, on Windows it is defined as HWND.
		 */
		const VISUAL_PLATFORM_VIEW getPlatformView(void);
        
        /**
		 * Updates the dimensions of the canvas according to the current dimensions of the platform view.
		 */
        void refreshCanvasRect(void);

		/**
		 * Returns the current dimensions of the canvas.
		 * @return The current dimensions of the canvas.
		 */
		PixelRect getCanvasRect(void);

		/**
		 * Ends the GL drawing by flushing, finishing and swaping the buffer content.
		 * Every executed drawing command should show its result on screen.
		 * The function is called at the end of each RenderMessage.
		 */
		void finishGLDrawing();
        
        /**
         * Returns the screen scale factor.
         * Retina displays on the Mac have a scren scale factor of 2.0.
         */
        float getScreenScaleFactor();

	private:

		/**
		 * The constructor. VisualGraphicsCore is a singleton class. The constructor is private. New instance of class can only be created internally.
		 */
		VisualGraphicsCore();

		/**
		 * The destructor. VisualGraphicsCore is a singleton class. The destructor is private. Instance of class can only be deleted internally.
		 */
		~VisualGraphicsCore();

		/**
		 * Copy constructor.
		 * @param other Another VisualGraphicsCore.
		 * @remarks Explicitely declared in private section and not implemented to enforce uniqueness of singleton pattern.
		 */
		VisualGraphicsCore(const VisualGraphicsCore& other);

		/**
		 * Assignment operator.
		 * @remarks Explicitely declared in private section and not implemented to enforce uniqueness of singleton pattern.
		 */			
		VisualGraphicsCore& operator=(const VisualGraphicsCore& other);

		/** VisualGraphicsCore is a singleton class. Pointer to private instance is handled internally. */
		static VisualGraphicsCore* theVisualGraphicsCore;

		/** The platform specific view.
		 * On Mac it is defined as NSView*. On Windows it is defined as HWND. 
		 */
		VISUAL_PLATFORM_VIEW platformView;
        
        /** The canvas rect is the area to draw in, measured in pixels. 
		 */
        PixelRect canvasRect;

#if TARGET_OS_MAC
        
        VisualView *visualView;
        
#endif
		
#if TARGET_OS_WIN
		/** The window RC. Windows-only. */
		HGLRC ctx;
		
		/** A handle to the current device. Windows-only. */
		HDC windowDC;
        
        /**
		 * Sets up the WGL canvas. Windows-only.
		 * @return Error number (0 = noErr).
		 */
		uint8 setupWGL();
        
        /**
		 * Cleans up wgl.
		 */
        void cleanUpWGL();
#endif

	};

}

#endif /* VisualGraphicsCore_h */
